#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_events.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <fcntl.h>

#include "jxd_global.h"
#include "jxd_ctrl.h"

/* For internal use only */
# define JXD_CTRL_UPRIGHT    0x10000
# define JXD_CTRL_UPLEFT     0x20000
# define JXD_CTRL_DOWNRIGHT  0x40000
# define JXD_CTRL_DOWNLEFT   0x80000


static int    loc_Volume = 50;

static int    loc_LastEventMask    = 0;
static int    loc_CurrEventMask    = 0;
static int    loc_CurrEventButtons = 0;
static u32    loc_LastTimeStamp    = 0;
static u32    loc_CurrTimeStamp    = 0;

static u16    loc_CurrMouse = 0;
static u16    loc_CurrMx = 0;
static u16    loc_CurrMy = 0;
static u16    loc_LastMx = 0;
static u16    loc_LastMy = 0;
static u32    loc_LastMouseTimeStamp = 0;

static u16    loc_LastLx = 0;
static u16    loc_LastLy = 0;

static int    loc_VolumeButtons = 0;
static int    loc_VolumePress   = 0;
static u32    loc_LastVolumeTimeStamp = 0;

# define JXD_MIN_TIME_VOLUME   300000
# define JXD_MIN_TIME_REPEAT   100000
#define JXD_MAX_TIME_CLICK     500000
# define JXD_MAX_TIME_GESTURE 1000000

# define JXD_CLICK_DELTA     10
# define JXD_CLICK_GESTURE   25

static inline int
jxdConvertMaskToButtons(int Mask)
{
  int Buttons = Mask & JXD_CTRL_MASK;
  if (Mask & JXD_CTRL_UPLEFT ) Buttons |= JXD_CTRL_UP|JXD_CTRL_LEFT;
  if (Mask & JXD_CTRL_UPRIGHT) Buttons |= JXD_CTRL_UP|JXD_CTRL_RIGHT;
  if (Mask & JXD_CTRL_DOWNLEFT ) Buttons |= JXD_CTRL_DOWN|JXD_CTRL_LEFT;
  if (Mask & JXD_CTRL_DOWNRIGHT) Buttons |= JXD_CTRL_DOWN|JXD_CTRL_RIGHT;
  return Buttons;
}

void
jxdTreatVolume(jxdCtrlData* c)
{
  if (c->Buttons & JXD_CTRL_VOLDOWN) {
    /* Already down ? */
    if (! (loc_VolumeButtons & JXD_CTRL_VOLDOWN)) {
      loc_LastVolumeTimeStamp = loc_CurrTimeStamp;
      loc_VolumeButtons |= JXD_CTRL_VOLDOWN;
      loc_VolumePress = 1;
      jxdDecreaseVolume();
    } else 
    if ((((loc_CurrTimeStamp - loc_LastVolumeTimeStamp) > JXD_MIN_TIME_VOLUME) && (loc_VolumePress == 1)) ||
        (((loc_CurrTimeStamp - loc_LastVolumeTimeStamp) > JXD_MIN_TIME_REPEAT) && (loc_VolumePress  > 1))) {
      loc_LastVolumeTimeStamp = loc_CurrTimeStamp;
      loc_VolumePress++;
      jxdDecreaseVolume();
    }

  } else
  if (c->Buttons & JXD_CTRL_VOLUP) {
    /* Already down ? */
    if (! (loc_VolumeButtons & JXD_CTRL_VOLUP)) {
      loc_LastVolumeTimeStamp = loc_CurrTimeStamp;
      loc_VolumeButtons |= JXD_CTRL_VOLUP;
      loc_VolumePress = 1;
      jxdIncreaseVolume();
    } else 
    if ((((loc_CurrTimeStamp - loc_LastVolumeTimeStamp) > JXD_MIN_TIME_VOLUME) && (loc_VolumePress == 1)) ||
        (((loc_CurrTimeStamp - loc_LastVolumeTimeStamp) > JXD_MIN_TIME_REPEAT) && (loc_VolumePress  > 1))) {
      loc_LastVolumeTimeStamp = loc_CurrTimeStamp;
      loc_VolumePress++;
      jxdIncreaseVolume();
    }

  } else {
    loc_VolumeButtons = 0;
  }
}

u32
jxdCtrlGetTimeStamp()
{
  return SDL_GetTicks() * 1000;
}

int  
jxdCtrlPeekBufferPositive(jxdCtrlData* c, int v)
{
  SDL_Event SDLEvent;    

  int KeyCode       = 0;
  int KeySym        = 0;
  int ButtonPress   = 0;
  int ButtonRelease = 0;
  int Mask = 0;
  int mouse_status = 0;
  int mouse_x = 0;
  int mouse_y = 0;
  int delta_x;
  int delta_y;
  int abs_delta_x;
  int abs_delta_y;

  memset(c, 0x0, sizeof(jxdCtrlData));
  loc_CurrTimeStamp = SDL_GetTicks() * 1000;

  c->Lx = loc_LastLx;
  c->Ly = loc_LastLy;
  c->Buttons   = loc_CurrEventButtons;
  c->TimeStamp = loc_CurrTimeStamp;

  loc_LastLx = 0;
  loc_LastLy = 0;

  if (SDL_PollEvent(&SDLEvent)) {

      if ((SDLEvent.type == SDL_KEYDOWN) ||
          (SDLEvent.type == SDL_KEYUP  )) {
        SDL_KeyboardEvent* Event = (SDL_KeyboardEvent*)(&SDLEvent);
  
        KeyCode = Event->keysym.scancode;
        KeySym  = Event->keysym.sym;

        if (SDLEvent.type == SDL_KEYDOWN) ButtonPress   = 1;
        else                              ButtonRelease = 1;
  
        Mask = 0;
  
        switch (KeyCode) {
          case JXD_UP        : Mask = JXD_CTRL_UP;
          break;
          case JXD_DOWN      : Mask = JXD_CTRL_DOWN;
          break;
          case JXD_LEFT      : Mask = JXD_CTRL_LEFT;
          break;
          case JXD_RIGHT     : Mask = JXD_CTRL_RIGHT;
          break;
  # if !defined(ANDROID_MODE)
          case JXD_UPLEFT    : Mask = JXD_CTRL_UPLEFT;
          break;
          case JXD_UPRIGHT   : Mask = JXD_CTRL_UPRIGHT;
          break;
          case JXD_DOWNLEFT  : Mask = JXD_CTRL_DOWNLEFT;
          break;
          case JXD_DOWNRIGHT : Mask = JXD_CTRL_DOWNRIGHT;
          break;
  # endif
          case JXD_A         : Mask = JXD_CTRL_SQUARE;
          break;
          case JXD_B         : Mask = JXD_CTRL_CIRCLE;
          break;
          case JXD_X         : Mask = JXD_CTRL_CROSS;
          break;
          case JXD_Y         : Mask = JXD_CTRL_TRIANGLE;
          break;
          case JXD_L         : Mask = JXD_CTRL_LTRIGGER;
          break;
          case JXD_R         : Mask = JXD_CTRL_RTRIGGER;
          break;
  # if ! defined(ANDROID_MODE)
          case JXD_FIRE      : Mask = JXD_CTRL_FIRE;
          break;
  # endif
          case JXD_START     : Mask = JXD_CTRL_START;
          break;
          case JXD_SELECT    : Mask = JXD_CTRL_SELECT;
          break;
  # if ! defined(ANDROID_MODE)
          case JXD_VOLUP     : Mask = JXD_CTRL_VOLUP;
          break;
          case JXD_VOLDOWN   : Mask = JXD_CTRL_VOLDOWN;
          break;
  # endif
        }
        loc_LastEventMask = loc_CurrEventMask;
        if (ButtonPress) {
          loc_CurrEventMask |= Mask;
        } else 
        if (ButtonRelease) {
          loc_CurrEventMask &= ~Mask;
        }
        loc_CurrEventButtons = jxdConvertMaskToButtons(loc_CurrEventMask);
        c->Buttons   = loc_CurrEventButtons;
        loc_LastTimeStamp = loc_CurrTimeStamp;
      }
  }

  mouse_status = SDL_GetMouseState( &mouse_x, &mouse_y );
  if (mouse_status) {
    if (! loc_CurrMouse) {
      loc_LastMx = mouse_x;
      loc_LastMy = mouse_y;
      loc_LastMouseTimeStamp = loc_CurrTimeStamp;
    }
    loc_CurrMouse = JXD_MOUSE_BTN_PRESS;
    loc_CurrMx = mouse_x;
    loc_CurrMy = mouse_y;

  } else {

    if (loc_CurrMouse == JXD_MOUSE_BTN_PRESS) {
      delta_x = loc_CurrMx - loc_LastMx;
      delta_y = loc_CurrMy - loc_LastMy;
      abs_delta_x = delta_x;
      if (abs_delta_x < 0) abs_delta_x = - abs_delta_x;
      abs_delta_y = delta_y;
      if (abs_delta_y < 0) abs_delta_y = - abs_delta_y;

      loc_CurrMouse = JXD_MOUSE_BTN_RELEASE;

      if ((abs_delta_x < JXD_CLICK_DELTA) &&
          (abs_delta_y < JXD_CLICK_DELTA) &&
          ((loc_CurrTimeStamp - loc_LastMouseTimeStamp) < JXD_MAX_TIME_CLICK)) {
        loc_CurrMouse |= JXD_MOUSE_CLICK;
      }
      if (((abs_delta_x > JXD_CLICK_GESTURE) ||
           (abs_delta_y > JXD_CLICK_GESTURE)) &&
          ((loc_CurrTimeStamp - loc_LastMouseTimeStamp) < JXD_MAX_TIME_GESTURE)) 
      {
        if (delta_x >  JXD_CLICK_GESTURE) loc_CurrMouse |= JXD_MOUSE_GST_RIGHT;
        if (delta_x < -JXD_CLICK_GESTURE) loc_CurrMouse |= JXD_MOUSE_GST_LEFT;
        if (delta_y >  JXD_CLICK_GESTURE) loc_CurrMouse |= JXD_MOUSE_GST_DOWN;
        if (delta_y < -JXD_CLICK_GESTURE) loc_CurrMouse |= JXD_MOUSE_GST_UP;
      }

    } else {
      loc_CurrMouse = 0;
      loc_CurrMx = 0;
      loc_CurrMy = 0;
    }
    loc_LastMx = 0;
    loc_LastMy = 0;
    loc_LastMouseTimeStamp = loc_CurrTimeStamp;
  }
  c->Mouse = loc_CurrMouse;
  c->Mx = loc_CurrMx;
  c->My = loc_CurrMy;

  jxdTreatVolume(c);

  return (c->Buttons != 0);
}

int
jxdCtrlReadBufferPositive(jxdCtrlData* c, int v)
{
  while (! jxdCtrlPeekBufferPositive(c, v));
  return 1;
}


int
jxdGetSoundVolume()
{
  return loc_Volume;
}

void
jxdDecreaseVolume()
{
  loc_Volume -= 2;
  if (loc_Volume < 0) loc_Volume = 0;
}

void
jxdIncreaseVolume()
{
  loc_Volume += 2;
  if (loc_Volume > 100) loc_Volume = 100;
}

