/*
 *  Copyright (C) 2012 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

# ifndef __JXD_CTRL_H__
# define __JXD_CTRL_H__

# ifdef __cplusplus
extern "C" {
# endif

typedef unsigned int u32;
typedef unsigned char u8;
typedef unsigned short u16;

#include "jxd_global.h"

# define JXD_ALL_BUTTON_MASK 0xFFFF

typedef struct jxdCtrlData {
  int    Buttons;
  u32    TimeStamp;
  short  Lx;   
  short  Ly;   
  u16    Mouse;
  u16    Mx;
  u16    My;

} jxdCtrlData;

typedef struct jxdCtrlData SceCtrlData;

# define JXD_CTRL_UP         0x00001
# define JXD_CTRL_RIGHT      0x00002
# define JXD_CTRL_DOWN       0x00004
# define JXD_CTRL_LEFT       0x00008
# define JXD_CTRL_TRIANGLE   0x00010
# define JXD_CTRL_CIRCLE     0x00020
# define JXD_CTRL_CROSS      0x00040
# define JXD_CTRL_SQUARE     0x00080
# define JXD_CTRL_SELECT     0x00100
# define JXD_CTRL_START      0x00200
# define JXD_CTRL_LTRIGGER   0x00400
# define JXD_CTRL_RTRIGGER   0x00800
# define JXD_CTRL_FIRE       0x01000
# define JXD_CTRL_VOLUP      0x02000
# define JXD_CTRL_VOLDOWN    0x04000
# define JXD_CTRL_MASK       0x07fff

# define JXD_CTRL_A          JXD_CTRL_SQUARE
# define JXD_CTRL_B          JXD_CTRL_CIRCLE
# define JXD_CTRL_Y          JXD_CTRL_TRIANGLE
# define JXD_CTRL_X          JXD_CTRL_CROSS

# define JXD_MOUSE_BTN_PRESS      0x0001
# define JXD_MOUSE_BTN_RELEASE    0x0002
# define JXD_MOUSE_CLICK          0x0010
# define JXD_MOUSE_GST_UP         0x0100
# define JXD_MOUSE_GST_DOWN       0x0200
# define JXD_MOUSE_GST_LEFT       0x0400
# define JXD_MOUSE_GST_RIGHT      0x0800


#if defined(WIZ_MODE)

//gp2x buttons codes

#define JXD_UP              (0)
#define JXD_DOWN            (4)
#define JXD_LEFT            (2)
#define JXD_RIGHT           (6)
#define JXD_UPLEFT          (1)
#define JXD_UPRIGHT         (7)
#define JXD_DOWNLEFT        (3)
#define JXD_DOWNRIGHT       (5)

#define JXD_A               (12)
#define JXD_B               (13)
#define JXD_X               (14)
#define JXD_Y               (15)
#define JXD_L               (10)
#define JXD_R               (11)
#define JXD_FIRE            (18)
#define JXD_START           (9)        
#define JXD_SELECT          (8)
#define JXD_VOLUP           (16)
#define JXD_VOLDOWN         (17)

#elif defined(PND_MODE)

#define JXD_UP              (111)
#define JXD_DOWN            (116)
#define JXD_LEFT            (113)
#define JXD_RIGHT           (114)

#define JXD_A               (110)
#define JXD_B               (115)
#define JXD_X               (117)
#define JXD_Y               (112)
#define JXD_L               (62)
#define JXD_R               (105)

#define JXD_START           (64)        
#define JXD_SELECT          (37)

#elif defined(ANDROID_MODE)

//jxd buttons codes

#define JXD_UP             (17)
#define JXD_DOWN           (18)
#define JXD_LEFT           (20)
#define JXD_RIGHT          (19)

#define JXD_B              (97)   // a : Circle
#define JXD_X              (98)   // b : Cross
#define JXD_Y              (99)   // c : Triangle
#define JXD_A              (100)  // d : Square
#define JXD_L              (101)  // e : L Trigger
#define JXD_R              (102)  // f : R Trigger

#define JXD_START          (13)   // start
#define JXD_SELECT         (32)   // select

#else
//some keys of the keyboard to emulate jxd

#define JXD_UPLEFT         79 //SDLK_KP7
#define JXD_UP             80 //SDLK_KP8
#define JXD_UPRIGHT        81 //SDLK_KP9

#define JXD_LEFT           83 //SDLK_KP4
#define JXD_RIGHT          85 //SDLK_KP6

#define JXD_DOWNLEFT       87 //SDLK_KP1
#define JXD_DOWN           88 //SDLK_KP2
#define JXD_DOWNRIGHT      89 //SDLK_KP3

# if 0
#define JXD_A              38 //SDLK_a
#define JXD_B              56 //SDLK_b
#define JXD_X              53 //SDLK_x
#define JXD_Y              29 //SDLK_y
# else
#define JXD_A              38 //SDLK_a
#define JXD_B              40 //SDLK_d
#define JXD_X              53 //SDLK_x
#define JXD_Y              25 //SDLK_w
# endif
#define JXD_L              46 //SDLK_l
#define JXD_R              27 //SDLK_r
#define JXD_FIRE           65 //SDLK_SPACE
#define JXD_START          36 //SDLK_RETURN
#define JXD_SELECT         39  //SDLK_s
#define JXD_VOLUP         86     //SDLK_KP_PLUS
#define JXD_VOLDOWN       82    //SDLK_KP_MINUS

#endif

#define JXD_NOEVENT -1

#define DELAY_KEY_FIRST 250
#define DELAY_KEY_REPEAT 80
    
  extern int  jxdCtrlReadBufferPositive(jxdCtrlData* c, int v);
  extern int  jxdCtrlPeekBufferPositive(jxdCtrlData* c, int v);
  extern u32  jxdCtrlGetTimeStamp();
  extern int  jxdCtrlReadKeyboardSym();

  extern int  jxdGetSoundVolume();
  extern void jxdDecreaseVolume();
  extern void jxdIncreaseVolume();

# if defined(JXD_MODE)
  extern int  jxdInsmodMMUhack();
  extern int  jxdRmmodMMUhack();
# endif

# ifdef __cplusplus
}
# endif

# endif
