/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <SDL/SDL.h>
#include <png.h>

#include "jxd_ctrl.h"
#include "jxd_sdl.h"
#include "jxd_global.h"
#include "jxd_play.h"
#include "jxd_mancala.h"
#include "jxd_menu.h"

  static int jxd_mancala_board_dirty = 1;

int
jxd_mancala_is_computer_move()
{
  if (MANCALA.mancala_game_over) return 0;

  if (MANCALA.mancala_game_mode == MANCALA_HUMAN_VS_COMPUTER) {
    return (MANCALA.computer_player == MANCALA.to_move);
  } else 
  if (MANCALA.mancala_game_mode == MANCALA_COMPUTER_VS_COMPUTER) {
    return 1;
  }
  return 0;
}

void
jxd_mancala_display_board()
{
  int  x_orig;
  int  y_orig;
  int  x_north;
  int  x_south;
  int  x_dst;
  int  y_dst;
  int  x;
  int  num_stones;

  int  square_width_pix = 750 / MANCALA_WIDTH;
  int  board_width_pix = MANCALA_WIDTH * square_width_pix;

  x_orig = ( JXD_SDL_SCREEN_WIDTH - board_width_pix ) / 2;
  y_orig = ( JXD_SDL_SCREEN_HEIGHT - (square_width_pix * 2)) / 2;

  jxd_sdl_blit_board();
  jxd_sdl_draw_rectangle(x_orig, y_orig-1, board_width_pix, (square_width_pix * 2)+1, MANCALA.board_color, 0);

  /* Draw board up : north */
  y_dst = y_orig;
  for (x = 0; x < MANCALA_WIDTH; x++) {
    x_dst = x_orig + (x * square_width_pix);
    jxd_sdl_blit_stone_xy(x_dst, y_dst, 0);
    num_stones = MANCALA.board_array[(MANCALA_NUM_PITS - 1) - x];
    jxd_sdl_blit_stone_xy(x_dst, y_dst, num_stones);
  }
  /* Draw board bottom : south */
  y_dst = y_dst + square_width_pix;
  for (x = 0; x < MANCALA_WIDTH; x++) {
    x_dst = x_orig + (x * square_width_pix);
    jxd_sdl_blit_stone_xy(x_dst, y_dst, 0);
    num_stones = MANCALA.board_array[x];
    jxd_sdl_blit_stone_xy(x_dst, y_dst, num_stones);
  }

  x_north = MANCALA.current_x[MANCALA_NORTH];
  x_south = MANCALA.current_x[MANCALA_SOUTH];

  jxd_sdl_blit_arrow_xy( x_orig + (x_north * square_width_pix) + square_width_pix/2, y_orig, 0);
  jxd_sdl_blit_arrow_xy( x_orig + (x_south * square_width_pix) + square_width_pix/2, y_orig + 2*square_width_pix, 1);

  jxd_sdl_draw_rectangle( x_orig, y_orig, 0, (square_width_pix * 2), MANCALA.board_color, 0);
}

void
jxd_mancala_display_status()
{
  char  buffer[128];

  int   x_orig = 10;
  int   y_orig = 10;

  /* Game mode */
  if (MANCALA.mancala_game_mode == MANCALA_HUMAN_VS_COMPUTER) strcpy(buffer, "Human vs CPU");
  else
  if (MANCALA.mancala_game_mode == MANCALA_HUMAN_VS_HUMAN)    strcpy(buffer, "Human vs Human");
  else                                                        strcpy(buffer, "CPU vs CPU");

  jxd_sdl_back3_print_ttf(x_orig, y_orig, buffer, JXD_MENU_YELLOW_COLOR);
  y_orig += jxd_sdl_get_ttf_height();

  if (MANCALA.mancala_game_over) {
    jxd_sdl_back3_print_ttf(x_orig, y_orig, "GAME OVER !", JXD_MENU_YELLOW_COLOR);
  } else
  if (jxd_mancala_is_computer_move()) {
    jxd_sdl_back3_print_ttf(x_orig, y_orig, "Waitting for CPU", JXD_MENU_GREEN_COLOR);
  }
  y_orig += jxd_sdl_get_ttf_height();

  if (MANCALA.to_move == MANCALA_NORTH) {
    jxd_sdl_back3_print_ttf(x_orig, y_orig, "North to play", JXD_MENU_NORTH_COLOR);
  } else {
    jxd_sdl_back3_print_ttf(x_orig, y_orig, "South to play", JXD_MENU_SOUTH_COLOR);
  }

  y_orig = JXD_SDL_SCREEN_HEIGHT - 2 * jxd_sdl_get_ttf_height();
  sprintf(buffer, "North %02d South %02d", MANCALA.score[MANCALA_NORTH], MANCALA.score[MANCALA_SOUTH]);
  jxd_sdl_back3_print_ttf(x_orig, y_orig, buffer, JXD_MENU_BLACK_COLOR);
}

void
jxd_mancala_refresh_screen()
{
  jxd_mancala_display_board();
  jxd_mancala_display_status();
}

void
jxd_mancala_mancala_left()
{
  int *p_current = &MANCALA.current_x[MANCALA.to_move];
  (*p_current)--;
  if ( *p_current < 0) *p_current = MANCALA_WIDTH - 1;
  jxd_mancala_board_dirty = 1;
}

void
jxd_mancala_mancala_right()
{
  int *p_current = &MANCALA.current_x[MANCALA.to_move];
  (*p_current)++;
  if ( *p_current >= MANCALA_WIDTH) *p_current = 0;
  jxd_mancala_board_dirty = 1;
}

int
jxd_mancala_goto(int r_x, int r_y)
{
  int  square_width_pix = 750 / MANCALA_WIDTH;
  int  board_width_pix = MANCALA_WIDTH * square_width_pix;

  int x_min = ( JXD_SDL_SCREEN_WIDTH - board_width_pix ) / 2;
  int y_min = ( JXD_SDL_SCREEN_HEIGHT - (square_width_pix * 2)) / 2;

  int x_max = x_min + board_width_pix;
  int y_max = y_min + (square_width_pix * 2);

  if ((r_x < x_min) || (r_x > x_max) || (r_y < y_min) || (r_y > y_max)) return 0;

  int x = ( r_x - x_min ) / square_width_pix;
  int y = ( r_y - y_min ) / square_width_pix;

  if (MANCALA.to_move == MANCALA_NORTH) {
    if (y == 0) { 
      MANCALA.current_x[MANCALA.to_move] = x;
      jxd_mancala_board_dirty = 1;
      return 1;
    }
  } else {
    if (y == 1) { 
      MANCALA.current_x[MANCALA.to_move] = x;
      jxd_mancala_board_dirty = 1;
      return 1;
    }
  }
  return 0;
}

void
jxd_mancala_new_game()
{
  jxd_global_reset();
  jxd_play_reset();
}

void
jxd_mancala_resign_game()
{
  if (MANCALA.mancala_game_over) return;
  MANCALA.mancala_game_over = 2;
}

void
jxd_mancala_game_over()
{
  if (MANCALA.mancala_game_over != 1) {
    jxd_play_compute_final_score();
  }
  jxd_mancala_refresh_screen();

  if (MANCALA.mancala_game_over == 1) return;
  MANCALA.mancala_game_over = 1;

# if 0 //LUDO: TO_BE_DONE !
  jxd_sdl_print(20, 120, "Game over press X to continue ...", JXD_MENU_RED_COLOR);
  jxd_sdl_flip();
  while (1)
  {
    SceCtrlData c;
    sceCtrlReadBufferPositive(&c, 1);
    if (c.Buttons & JXD_CTRL_CROSS) break;
  }
  jxd_kbd_wait_no_button();
# endif
  jxd_mancala_board_dirty = 1;
}

void
jxd_mancala_play()
{
  if (MANCALA.mancala_game_over) return;

  jxd_play_player_move();
  jxd_mancala_board_dirty = 1;
}

void
jxd_mancala_main_loop()
{
  jxdCtrlData c;
  long        new_pad = 0;
  long        old_pad;
  int         last_time;

  int new_Lx = 0;
  int new_Ly = 0;

  old_pad   = 0;
  last_time = 0;

  jxd_main_menu();

  while (1) {

    do {

      if (jxd_mancala_board_dirty) {
        jxd_mancala_board_dirty = 0;
        jxd_mancala_refresh_screen();
        jxd_sdl_flip();
      }

      if (jxd_mancala_is_computer_move()) {

        if (jxd_play_computer_move()) {
          jxd_mancala_refresh_screen();
          jxd_sdl_flip();

          if (MANCALA.mancala_game_mode == MANCALA_COMPUTER_VS_COMPUTER) {
            MANCALA.computer_player = ! MANCALA.computer_player;
          }
        }
      }

      if (jxd_play_is_game_over()) {
        /* Game OVER ! */
        jxd_mancala_game_over();
      }

    } while (jxd_mancala_board_dirty);

    if (new_pad & (JXD_CTRL_SQUARE|JXD_CTRL_CROSS|JXD_CTRL_CIRCLE|JXD_CTRL_TRIANGLE)) {
      jxd_kbd_wait_no_button();
    }

    while (1)
    {
      jxdCtrlPeekBufferPositive(&c, 1);
      if (c.Mouse) break;

      if (c.Buttons)  break;
      if (MANCALA.mancala_game_mode == MANCALA_COMPUTER_VS_COMPUTER) {
        break;
      }
    }

    if ((c.Mouse & (JXD_MOUSE_CLICK|JXD_MOUSE_BTN_PRESS))) {
      if (jxd_mancala_goto(c.Mx, c.My)) {
        jxd_mancala_play();
        continue;
      }
    }

    new_pad = c.Buttons;

    if (new_pad) {
      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > JXD_MANCALA_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
      } else continue;
    }

    if((new_pad & JXD_CTRL_SELECT) == JXD_CTRL_SELECT) {
      jxd_main_menu();
      jxd_mancala_board_dirty = 1;
    }  else
    if ((new_pad & (JXD_CTRL_CROSS|JXD_CTRL_CIRCLE))) {
      jxd_mancala_play();
    } else
    if ((new_pad == JXD_CTRL_LEFT)) {
      jxd_mancala_mancala_left();
    } else
    if ((new_pad == JXD_CTRL_RIGHT)) {
      jxd_mancala_mancala_right();
    }
  }
}

int 
SDL_main(int argc,char *argv[])
{
  jxd_sdl_init();
  jxd_global_init();
  jxd_play_init();

  jxd_mancala_main_loop();

  jxd_sdl_exit(0);

  return 0;
}

