/*
 *  Copyright (C) 2006 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <fcntl.h>

#include "SDL.h"

#include "jxd_ctrl.h"
#include "jxd_global.h"
#include "jxd_kbd.h"
#include "jxd_sdl.h"
#include "jxd_menu.h"
#include "jxd_menu_help.h"

# define MAX_HELP_LINE    4096 

  static char* jxd_help[MAX_HELP_LINE];
  static int   jxd_help_size    = -1;
  static int   jxd_help_current = 0;
  static int   jxd_help_line_by_page = 10;
  static int   jxd_help_char_by_line = 40;

static void
jxd_initialize_help(void)
{
  char  FileName[MAX_PATH+1];

  char  Buffer[512];
  char *Scan;
  FILE* FileDesc;

  /* Already done ? */
  if (jxd_help_size > 0) return; 

  int font_width  = jxd_sdl_get_ttf_width();
  int font_heigth = jxd_sdl_get_ttf_height();

  jxd_help_char_by_line = (JXD_SDL_SCREEN_WIDTH  - 2 * font_width ) / font_width;
  jxd_help_line_by_page = (JXD_SDL_SCREEN_HEIGHT                  ) / font_heigth;

  strcpy(FileName, "./help.txt");
  FileDesc = fopen(FileName, "r");

  jxd_help_current = 0;

  if (FileDesc == (FILE *)0 ) {
    jxd_help[0] = strdup( "no help file found !");
    jxd_help_size = 1;
    return;
  }

  jxd_help_size = 0;
  while (fgets(Buffer,512, FileDesc) != (char *)0) {

    Scan = strchr(Buffer,'\n');
    if (Scan) *Scan = '\0';
    /* For this #@$% of windows ! */
    Scan = strchr(Buffer,'\r');
    if (Scan) *Scan = '\0';

    jxd_help[jxd_help_size++] = strdup(Buffer);
    if (jxd_help_size >= MAX_HELP_LINE) break;
  }
  fclose(FileDesc);
}

static void 
jxd_display_screen_help( int blit_bg )
{
  char buffer[512];

  int help_line = 0;
  int index     = 0;

  int x       = 0;
  int y       = 0;
  int y_step  = 0;

  if (blit_bg) {
    jxd_sdl_blit_help();
  }

  x      = jxd_sdl_get_ttf_width();
  y      = 0;
  y_step = jxd_sdl_get_ttf_height();

  help_line = jxd_help_current;
  index     = 0;
  
  while ((index < jxd_help_line_by_page) && (help_line < jxd_help_size))  {
    strcpy(buffer, jxd_help[help_line]);
    string_fill_with_space(buffer, jxd_help_char_by_line);
    jxd_sdl_print_ttf(x, y, buffer, JXD_MENU_SEL_COLOR);
    y += y_step;
    index++;
    help_line++;
  }

  if (index != jxd_help_line_by_page) {
    buffer[0]=0;
    string_fill_with_space(buffer, jxd_help_char_by_line);
    while (index < jxd_help_line_by_page) {
      jxd_sdl_print_ttf(x, y, buffer, JXD_MENU_SEL_COLOR);
      y += y_step;
      index++;
    }
  }
}

int 
jxd_help_menu()
{
  jxdCtrlData c;
  long        new_pad;
  long        old_pad;
  int         last_time;
  int         end_menu;
  int         blit_bg;

  jxd_kbd_wait_no_button();

  jxd_initialize_help();

  old_pad   = 0;
  last_time = 0;
  end_menu  = 0;
  blit_bg   = 1;

  while (! end_menu)
  {
    jxd_display_screen_help( blit_bg );
    jxd_sdl_flip();

    while (1)
    {
      jxdCtrlPeekBufferPositive(&c, 1);
      new_pad = c.Buttons;
# if defined(MOUSE_MODE)
      if (c.Mouse) break;
# endif

      if ((old_pad != new_pad) || ((c.TimeStamp - last_time) > JXD_MENU_MIN_TIME)) {
        last_time = c.TimeStamp;
        old_pad = new_pad;
        break;
      }
    }

# if defined(MOUSE_MODE)
    if (c.My > (JXD_SDL_SCREEN_HEIGHT - 20)) {
      if (c.Mouse & JXD_MOUSE_BTN_PRESS) new_pad = JXD_CTRL_DOWN;
    } else
    if (c.My < 20) {
      if (c.Mouse & JXD_MOUSE_BTN_PRESS) new_pad = JXD_CTRL_UP;
    } else
    if (c.Mouse & (JXD_MOUSE_GST_LEFT|JXD_MOUSE_GST_UP)) new_pad = JXD_CTRL_LEFT;
    else
    if (c.Mouse & (JXD_MOUSE_GST_RIGHT|JXD_MOUSE_GST_DOWN)) new_pad = JXD_CTRL_RIGHT;
# endif

    if (new_pad) {
      blit_bg = 1;
    }

    if(new_pad & JXD_CTRL_SELECT) {
      /* Back to Main menu */
      end_menu = 1;
    } else
    if(new_pad & JXD_CTRL_UP) {
      if (jxd_help_current > 0) jxd_help_current--;
    } else
    if(new_pad & JXD_CTRL_DOWN) {
      if ((jxd_help_current + 1) < jxd_help_size) jxd_help_current++;
    } else
    if(new_pad & JXD_CTRL_LEFT) {
      if (jxd_help_current > jxd_help_line_by_page) jxd_help_current -= jxd_help_line_by_page;
      else                                      jxd_help_current  = 0;
    } else
    if(new_pad & JXD_CTRL_RIGHT) {
      if ((jxd_help_current + jxd_help_line_by_page + 1) < jxd_help_size) jxd_help_current += jxd_help_line_by_page;
    }
  }
 
  jxd_kbd_wait_no_button();

  return 1;
}

