/*
 *  Copyright (C) 2012 Ludovic Jacomme (ludovic.jacomme@gmail.com)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <SDL/SDL_ttf.h>
#include <png.h>

#include "jxd_ctrl.h"
#include "jxd_sdl.h"
#include "jxd_global.h"

  extern unsigned char jxd_btm_6x10[];
  extern unsigned char jxd_btm_8x8[];
  extern unsigned char jxd_btm_8x10[];
  extern unsigned char jxd_btm_8x12[];
  extern unsigned char jxd_btm_8x14[];
  extern unsigned char jxd_btm_8x16[];

  unsigned char *jxd_btm        = jxd_btm_8x14;
  int            jxd_btm_width  = 8; 
  int            jxd_btm_height = 14; 

  TTF_Font      *jxd_ttf        = NULL;
  int            jxd_ttf_size   = 16;
  int            jxd_ttf_width  = 8; 
  int            jxd_ttf_height = 8; 
  char           jxd_ttf_filename[] = "font.ttf";


  SDL_Surface *background_surface;
  SDL_Surface *help_surface;

  SDL_Surface *back2_surface;
  SDL_Surface *back3_surface;
  SDL_Surface *splash_surface;

  SDL_Surface *back_surface;


  SDL_Surface *stones_surface[MANCALA_MAX_STONES_IMG + 1];
 
  SDL_Surface *arrows_surface[2];

static void
jxd_sdl_init_ttf()
{
  TTF_Init();
}

int
jxd_sdl_get_ttf_width()
{
  return jxd_ttf_width;
}

int
jxd_sdl_get_ttf_height()
{
  return jxd_ttf_height;
}


static void
jxd_sdl_destroy_ttf()
{
  if (jxd_ttf) {
	  TTF_CloseFont( jxd_ttf ); 
	  TTF_Quit(); 
    jxd_ttf = NULL;
  }
}

void
jxd_sdl_select_font_tt( int size )
{
  if (jxd_ttf) {
    TTF_CloseFont( jxd_ttf );
    jxd_ttf = NULL;
  }
  jxd_ttf = TTF_OpenFont( jxd_ttf_filename, size );
  TTF_SizeText( jxd_ttf, "W", &jxd_ttf_width, &jxd_ttf_height );
}

int
jxd_sdl_print_ttf(int x,int y,const char *str, int color)
{
  SDL_Surface* ttf_surface = 0;
  SDL_Color    ttf_color = { 0, 0, 0 };
  Uint32       ttf_pixel = color;
  SDL_GetRGB( ttf_pixel, back_surface->format, &ttf_color.r, &ttf_color.g, &ttf_color.b );

  ttf_surface = TTF_RenderText_Blended( jxd_ttf, str, ttf_color );

  if (ttf_surface) {
    int w = ttf_surface->w;
    jxd_sdl_back2_rectangle( x, y, ttf_surface->w, ttf_surface->h );
    SDL_Rect dst;
    dst.x = x; dst.y = y; dst.w = 0; dst.h = 0;
    SDL_BlitSurface( ttf_surface, NULL, back_surface, &dst );
    SDL_FreeSurface( ttf_surface );
    return w;
  }
}

int
jxd_sdl_back3_print_ttf(int x,int y,const char *str, int color)
{
  SDL_Surface* ttf_surface = 0;
  SDL_Color    ttf_color = { 0, 0, 0 };
  Uint32       ttf_pixel = color;
  SDL_GetRGB( ttf_pixel, back_surface->format, &ttf_color.r, &ttf_color.g, &ttf_color.b );

  ttf_surface = TTF_RenderText_Blended( jxd_ttf, str, ttf_color );

  if (ttf_surface) {
    int w = ttf_surface->w;
    jxd_sdl_back3_rectangle( x, y, ttf_surface->w, ttf_surface->h );
    SDL_Rect dst;
    dst.x = x; dst.y = y; dst.w = 0; dst.h = 0;
    SDL_BlitSurface( ttf_surface, NULL, back_surface, &dst );
    SDL_FreeSurface( ttf_surface );
    return w;
  }
}


int
jxd_sdl_get_btm_height()
{
  return jxd_btm_height;
}

int
jxd_sdl_get_btm_width()
{
  return jxd_btm_width;
}


uint
jxd_sdl_rgb(uchar R, uchar G, uchar B)
{
  return SDL_MapRGB(back_surface->format, R,G,B);
}

ushort *
jxd_sdl_get_vram_addr(uint x, uint y)
{
  ushort *vram = (ushort *)back_surface->pixels;
  return vram + x + (y*JXD_LINE_SIZE);
}

void 
jxd_sdl_put_char_btm(int x, int y, int color, int bgcolor, uchar c, int drawfg, int drawbg)
{
  int cx;
  int cy;
  int bmask;
  int index;

  ushort *vram = (ushort *)jxd_sdl_get_vram_addr(x, y);

  index = ((ushort)c) * jxd_btm_height;
  for (cy=0; cy< jxd_btm_height; cy++) {
    bmask = 1 << ( jxd_btm_width - 1);
    for (cx=0; cx< jxd_btm_width; cx++) {
      if (jxd_btm[index] & bmask) {
        if (drawfg) vram[cx + cy * JXD_LINE_SIZE] = color;
      } else {
        if (drawbg) vram[cx + cy * JXD_LINE_SIZE] = bgcolor;
      }
      bmask = bmask >> 1;
    }
    index++;
  }
}

void 
jxd_sdl_print_btm(int x,int y, const char *str, int color)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    jxd_sdl_put_char_btm(x, y, color, 0, str[index], 1, 0);
    x += jxd_btm_width;
    if (x >= (JXD_SDL_SCREEN_WIDTH - jxd_btm_width)) {
      x = x0; y += jxd_btm_height;
    }
    if (y >= (JXD_SDL_SCREEN_HEIGHT - jxd_btm_width)) break;
  }
}

void 
jxd_sdl_print_btm_shadow(int x,int y, char *str, int color)
{
    jxd_sdl_print_btm(x+1,y+1,str,0);
    jxd_sdl_print_btm(x,y,str,color);

}

void
jxd_sdl_clear_screen(int color)
{
  int x; int y;
  ushort *vram = jxd_sdl_get_vram_addr(0,0);
  
  for (y = 0; y < JXD_SDL_SCREEN_HEIGHT; y++) {
    for (x = 0; x < JXD_SDL_SCREEN_WIDTH; x++) {
      vram[x + (y*JXD_LINE_SIZE)] = color;
    }
  }
}

void
jxd_sdl_black_screen()
{
  SDL_FillRect(back_surface,NULL,SDL_MapRGB(back_surface->format,0x0,0x0,0x0));
  SDL_Flip(back_surface);
  SDL_FillRect(back_surface,NULL,SDL_MapRGB(back_surface->format,0x0,0x0,0x0));
  SDL_Flip(back_surface);
}


void 
jxd_sdl_draw_rectangle(int x, int y, int w, int h, int border, int mode) 
{
  ushort *vram = (ushort *)jxd_sdl_get_vram_addr(x, y);
  int xo, yo;
  // clip
  if ((x + w) >= JXD_SDL_SCREEN_WIDTH ) w = JXD_SDL_SCREEN_WIDTH  - x - 1;
  if ((y + h) >= JXD_SDL_SCREEN_HEIGHT) h = JXD_SDL_SCREEN_HEIGHT - y - 1;

  if (mode == JXD_SDL_XOR) {
    for (xo = 0; xo < w; xo++) {
      vram[xo] ^=  border;
      vram[xo + h * JXD_LINE_SIZE] ^=  border;
    }
    for (yo = 0; yo < h; yo++) {
      vram[yo * JXD_LINE_SIZE] ^=  border;
      vram[w + yo * JXD_LINE_SIZE] ^=  border;
    }
    vram[w + h * JXD_LINE_SIZE] ^=  border;
  } else {
    for (xo = 0; xo < w; xo++) {
      vram[xo] =  border;
      vram[xo + h * JXD_LINE_SIZE] =  border;
    }
    for (yo = 0; yo < h; yo++) {
      vram[yo * JXD_LINE_SIZE] =  border;
      vram[w + yo * JXD_LINE_SIZE] =  border;
    }
    vram[w + h * JXD_LINE_SIZE] =  border;
  }
}

void 
jxd_sdl_fill_rectangle(int x, int y, int w, int h, int color, int mode)
{
  ushort *vram  = (ushort *)jxd_sdl_get_vram_addr(x, y);
  int xo, yo;

  // clip
  if ((x + w) >= JXD_SDL_SCREEN_WIDTH ) w = JXD_SDL_SCREEN_WIDTH  - x - 1;
  if ((y + h) >= JXD_SDL_SCREEN_HEIGHT) h = JXD_SDL_SCREEN_HEIGHT - y - 1;

  if (mode == JXD_SDL_XOR) {
    for (xo = 0; xo <= w; xo++) {
      for (yo = 0; yo <= h; yo++) {
        if ( ((xo == 0) && ((yo == 0) || (yo == h))) ||
             ((xo == w) && ((yo == 0) || (yo == h))) ) {
          /* Skip corner */
        } else {
          vram[xo + yo * JXD_LINE_SIZE] ^=  color;
        }
      }
    }
  } else {
    for (xo = 0; xo <= w; xo++) {
      for (yo = 0; yo <= h; yo++) {
        vram[xo + yo * JXD_LINE_SIZE] =  color;
      }
    }
  }
}

static int
jxd_sdl_get_back2_color(int x, int y)
{
  uchar *back2 = (uchar *)back2_surface->pixels;
  int bytes_per_pixels = back2_surface->format->BytesPerPixel;
  int pitch            = back2_surface->pitch;
  Uint8 r = back2[0 + (y * pitch) + (x * bytes_per_pixels)];
  Uint8 g = back2[1 + (y * pitch) + (x * bytes_per_pixels)];
  Uint8 b = back2[2 + (y * pitch) + (x * bytes_per_pixels)];
	int color = jxd_sdl_rgb(r, g, b);

  return color;
}

static int
jxd_sdl_get_back3_color(int x, int y)
{
  uchar *back3 = (uchar *)back3_surface->pixels;
  int bytes_per_pixels = back3_surface->format->BytesPerPixel;
  int pitch            = back3_surface->pitch;
  Uint8 r = back3[0 + (y * pitch) + (x * bytes_per_pixels)];
  Uint8 g = back3[1 + (y * pitch) + (x * bytes_per_pixels)];
  Uint8 b = back3[2 + (y * pitch) + (x * bytes_per_pixels)];
	int color = jxd_sdl_rgb(r, g, b);

  return color;
}


void 
jxd_sdl_back2_rectangle(int x, int y, int w, int h)
{
  if (! back2_surface) {
    jxd_sdl_fill_rectangle(x, y, w, h, 0x0, 0);
    return;
  }

  ushort *vram  = (ushort *)jxd_sdl_get_vram_addr(x, y);

  int xo, yo;
  for (xo = 0; xo <= w; xo++) {
    for (yo = 0; yo <= h; yo++) {
      vram[xo + yo * JXD_LINE_SIZE] = jxd_sdl_get_back2_color(x + xo, y + yo);
    }
  }
}

void 
jxd_sdl_back3_rectangle(int x, int y, int w, int h)
{
  if (! back3_surface) {
    jxd_sdl_fill_rectangle(x, y, w, h, 0x0, 0);
    return;
  }

  ushort *vram  = (ushort *)jxd_sdl_get_vram_addr(x, y);

  int xo, yo;
  for (xo = 0; xo <= w; xo++) {
    for (yo = 0; yo <= h; yo++) {
      vram[xo + yo * JXD_LINE_SIZE] = jxd_sdl_get_back3_color(x + xo, y + yo);
    }
  }
}

void 
jxd_sdl_put_char(int x, int y, int color, int bgcolor, uchar c, int drawfg, int drawbg)
{
  int cx;
  int cy;
  int b;
  int index;

  ushort *vram = (ushort *)jxd_sdl_get_vram_addr(x, y);
  index = ((ushort)c) * jxd_btm_height;

  for (cy=0; cy< jxd_btm_height; cy++) {
    b = 1 << (jxd_btm_width - 1);
    for (cx=0; cx< jxd_btm_width; cx++) {
      if (jxd_btm[index] & b) {
        if (drawfg) vram[cx + cy * JXD_LINE_SIZE] = color;
      } else {
        if (drawbg) vram[cx + cy * JXD_LINE_SIZE] = bgcolor;
      }
      b = b >> 1;
    }
    index++;
  }
}

void 
jxd_sdl_back2_put_char_btm(int x, int y, int color, uchar c)
{
  int cx;
  int cy;
  int bmask;
  int index;

  if (! back2_surface) {
    jxd_sdl_put_char_btm(x, y, color, 0x0, c, 1, 1);
    return;
  }

  ushort *vram  = (ushort *)jxd_sdl_get_vram_addr(x, y);

  index = ((ushort)c) * jxd_btm_height;
  for (cy=0; cy< jxd_btm_height; cy++) {
    bmask = 1 << ( jxd_btm_width - 1);
    for (cx=0; cx< jxd_btm_width; cx++) {
      if (jxd_btm[index] & bmask) {
        vram[cx + cy * JXD_LINE_SIZE] = color;
      } else {
        vram[cx + cy * JXD_LINE_SIZE] = jxd_sdl_get_back2_color(x + cx, y + cy);
      }
      bmask = bmask >> 1;
    }
    index++;
  }
}

unsigned char
jxd_convert_utf8_to_iso_8859_1(unsigned char c1, unsigned char c2)
{
  unsigned char res = 0;
  if (c1 == 0xc2) res = c2;
  else
  if (c1 == 0xc3) res = c2 | 0x40;
  return res;
}

void 
jxd_sdl_back3_put_char(int x, int y, int color, uchar c)
{
  int cx;
  int cy;
  int bmask;
  int index;

  if (! back3_surface) {
    jxd_sdl_put_char(x, y, color, 0x0, c, 1, 1);
    return;
  }

  ushort *vram  = (ushort *)jxd_sdl_get_vram_addr(x, y);

  index = ((ushort)c) * jxd_btm_height;

  for (cy=0; cy< jxd_btm_height; cy++) {
    bmask = 1 << (jxd_btm_width - 1);
    for (cx=0; cx< jxd_btm_width; cx++) {
      if (jxd_btm[index] & bmask) {
        vram[cx + cy * JXD_LINE_SIZE] = color;
      } else {
        vram[cx + cy * JXD_LINE_SIZE] = jxd_sdl_get_back3_color(x + cx, y + cy);
      }
      bmask = bmask >> 1;
    }
    index++;
  }
}

void 
jxd_sdl_fill_print_btm(int x,int y,const char *str, int color, int bgcolor)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    uchar c = str[index];
    if ((c == 0xc2) || (c == 0xc3)) {
      uchar new_c = jxd_convert_utf8_to_iso_8859_1(c, str[index+1]);
      if (new_c) { c = new_c; index++; }
    }
    jxd_sdl_put_char_btm(x, y, color, bgcolor, c, 1, 1);
    x += jxd_btm_width;
    if (x >= (JXD_SDL_SCREEN_WIDTH - jxd_btm_width)) {
      x = x0; y++;
    }
    if (y >= (JXD_SDL_SCREEN_HEIGHT - jxd_btm_width)) break;
  }
}

void
jxd_sdl_back2_print(int x,int y,const char *str, int color)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    uchar c = str[index];
    if ((c == 0xc2) || (c == 0xc3)) {
      uchar new_c = jxd_convert_utf8_to_iso_8859_1(c, str[index+1]);
      if (new_c) { c = new_c; index++; }
    }
    jxd_sdl_back2_put_char_btm(x, y, color, c);
    x += jxd_btm_width;
    if (x >= (JXD_SDL_SCREEN_WIDTH - jxd_btm_width)) {
      x = x0; y++;
    }
    if (y >= (JXD_SDL_SCREEN_HEIGHT - jxd_btm_width)) break;
  }
}

void
jxd_sdl_back3_print(int x,int y,const char *str, int color)
{
  int index;
  int x0 = x;

  for (index = 0; str[index] != '\0'; index++) {
    uchar c = str[index];
    if ((c == 0xc2) || (c == 0xc3)) {
      uchar new_c = jxd_convert_utf8_to_iso_8859_1(c, str[index+1]);
      if (new_c) { c = new_c; index++; }
    }
    jxd_sdl_back3_put_char(x, y, color, c);
    x += jxd_btm_width;
    if (x >= (JXD_SDL_SCREEN_WIDTH - jxd_btm_width)) {
      x = x0; y++;
    }
    if (y >= (JXD_SDL_SCREEN_HEIGHT - jxd_btm_width)) break;
  }
}


void
jxd_sdl_load_background()
{
  background_surface = IMG_Load("./background.png");
}

void
jxd_sdl_load_help()
{
  help_surface = IMG_Load("./help.png");
}

void
jxd_sdl_load_stones()
{
  char filename[128];
  int  index;

  for (index = 0; index <= MANCALA_MAX_STONES_IMG; index++) {
    sprintf(filename, "./images/stone%02d-125x125.png", index);
    stones_surface[index] = IMG_Load(filename);
    if (! stones_surface[index]) {
      fprintf(stderr, "unable to load %s\n", filename);
      jxd_sdl_exit(1);
    }
  }
}

void
jxd_sdl_load_arrows()
{
  arrows_surface[0] = IMG_Load("./images/down-12x12.png");
  arrows_surface[1] = IMG_Load("./images/up-12x12.png");
  if (! arrows_surface[0] || ! arrows_surface[1]) {
    fprintf(stderr, "unable to load up/down arrows png\n");
    jxd_sdl_exit(1);
  }
}

void
jxd_sdl_blit_stone_xy(int x, int y, int num_stones)
{
  static int first = 1;

  if (first && (stones_surface[0] == NULL)) {
    jxd_sdl_load_stones();
    first = 0;
  }
	SDL_Rect src_rect;
  SDL_Rect dst_rect;
  src_rect.x = 0;
  src_rect.y = 0;
  src_rect.w = 125;
  src_rect.h = 125;

  dst_rect.x = x;
  dst_rect.y = y;
  dst_rect.w = 125;
  dst_rect.h = 125;

  if (num_stones > MANCALA_MAX_STONES_IMG) {
    while (num_stones > MANCALA_MAX_STONES_IMG) {
	    SDL_BlitSurface(stones_surface[MANCALA_MAX_STONES_IMG], &src_rect, back_surface, &dst_rect);
      dst_rect.x++;
      dst_rect.y++;
      num_stones -= MANCALA_MAX_STONES_IMG;
    }
  }
	SDL_BlitSurface(stones_surface[num_stones], &src_rect, back_surface, &dst_rect);
# if 0
  sprintf(buffer, "%d", num_stones);
  jxd_sdl_print(x + 40, y + 40, buffer, 0x0);
# endif
}

void
jxd_sdl_blit_arrow_xy(int x, int y, int arrow)
{
  static int first = 1;

  if (first && (arrows_surface[0] == NULL)) {
    jxd_sdl_load_arrows();
    first = 0;
  }
	SDL_Rect src_rect;
  SDL_Rect dst_rect;
  src_rect.x = 0;
  src_rect.y = 0;
  src_rect.w = 12;
  src_rect.h = 12;

  dst_rect.x = x - 6;
  if (!arrow) dst_rect.y = y - 12; /* Down */
  else        dst_rect.y = y;

  dst_rect.w = 12;
  dst_rect.h = 12;
  
  if ((arrow < 2) && (arrows_surface[arrow] != NULL)) {
	  SDL_BlitSurface(arrows_surface[arrow], &src_rect, back_surface, &dst_rect);
  } else {
    jxd_sdl_fill_rectangle(dst_rect.x, dst_rect.y, dst_rect.w, dst_rect.h, 0x0, 0);
  }
}

void
jxd_sdl_blit_background()
{
  static int first = 1;

  if (first && (background_surface == NULL)) {
    jxd_sdl_load_background();
    first = 0;
  }
  back2_surface = background_surface;

  if (back2_surface != NULL) {
	  SDL_BlitSurface(back2_surface, NULL, back_surface, NULL);
  } else {
    jxd_sdl_clear_screen(jxd_sdl_rgb(0x00, 0x00, 0x00));
  }
}

void
jxd_sdl_blit_help()
{
  static int first = 1;

  if (first && (help_surface == NULL)) {
    jxd_sdl_load_help();
    first = 0;
  }
  back2_surface = help_surface;

  if (back2_surface != NULL) {
	  SDL_BlitSurface(back2_surface, NULL, back_surface, NULL);
  } else {
    jxd_sdl_clear_screen(jxd_sdl_rgb(0x00, 0x00, 0x00));
  }
}


void
jxd_sdl_load_board()
{
  back3_surface = IMG_Load("./board.png");
}

void
jxd_sdl_load_splash()
{
  splash_surface = IMG_Load("./splash.png");
}

void
jxd_sdl_blit_board()
{
  static int first = 1;

  if (first && (back3_surface == NULL)) {
    jxd_sdl_load_board();
    first = 0;
  }
  if (back3_surface != NULL) {
	  SDL_BlitSurface(back3_surface, NULL, back_surface, NULL);
  } else {
    jxd_sdl_clear_screen(jxd_sdl_rgb(0x00, 0x00, 0x00));
  }
}

void
jxd_sdl_blit_splash()
{
  static int first = 1;

  if (first && (splash_surface == NULL)) {
    jxd_sdl_load_splash();
    first = 0;
  }
  if (splash_surface != NULL) {
	  SDL_BlitSurface(splash_surface, NULL, back_surface, NULL);
  } else {
    jxd_sdl_clear_screen(jxd_sdl_rgb(0x00, 0x00, 0x00));
  }
}

void
jxd_sdl_display_splash()
{
  int index = 0;
  char buffer[128];
  jxdCtrlData c;

  sprintf(buffer, "v%s", MANCALA_VERSION);

  int x = (JXD_SDL_SCREEN_WIDTH - 10 - (strlen(buffer) * jxd_sdl_get_ttf_width()));
  int y = JXD_SDL_SCREEN_HEIGHT - 10 - jxd_sdl_get_ttf_height();
  int col = jxd_sdl_rgb(0xa0, 0xa0, 0xa0);


  jxd_sdl_blit_splash();
  jxd_sdl_print_ttf(x, y, buffer, col);
  jxd_sdl_flip();

  while (index < 10) {
    SDL_Delay( 150 );
    jxdCtrlPeekBufferPositive(&c, 1);
    if (c.Buttons & (JXD_CTRL_START|JXD_CTRL_CROSS)) break;
    index++;
  }
}

void
jxd_sdl_flip(void)
{
  SDL_Flip(back_surface);
}

#define  systemRedShift      (back_surface->format->Rshift)
#define  systemGreenShift    (back_surface->format->Gshift)
#define  systemBlueShift     (back_surface->format->Bshift)
#define  systemRedMask       (back_surface->format->Rmask)
#define  systemGreenMask     (back_surface->format->Gmask)
#define  systemBlueMask      (back_surface->format->Bmask)

int
jxd_sdl_save_png(char *filename)
{
  int w = JXD_SDL_SCREEN_WIDTH;
  int h = JXD_SDL_SCREEN_HEIGHT;
  u8* pix = (u8*)jxd_sdl_get_vram_addr(0,0);
  u8 writeBuffer[800 * 3];

  FILE *fp = fopen(filename,"wb");

  if(!fp) return;
  
  png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
                                                NULL,
                                                NULL,
                                                NULL);
  if(!png_ptr) {
    fclose(fp);
    return;
  }

  png_infop info_ptr = png_create_info_struct(png_ptr);

  if(!info_ptr) {
    png_destroy_write_struct(&png_ptr,NULL);
    fclose(fp);
    return;
  }

  png_init_io(png_ptr,fp);

  png_set_IHDR(png_ptr,
               info_ptr,
               w,
               h,
               8,
               PNG_COLOR_TYPE_RGB,
               PNG_INTERLACE_NONE,
               PNG_COMPRESSION_TYPE_DEFAULT,
               PNG_FILTER_TYPE_DEFAULT);

  png_write_info(png_ptr,info_ptr);

  u8 *b = writeBuffer;

  int sizeX = w;
  int sizeY = h;
  int y;
  int x;

  u16 *p = (u16 *)pix;
  for(y = 0; y < sizeY; y++) {
     for(x = 0; x < sizeX; x++) {
      u16 v = p[x];

      *b++ = ((v & systemRedMask  ) >> systemRedShift  ) << 3; // R
      *b++ = ((v & systemGreenMask) >> systemGreenShift) << 2; // G 
      *b++ = ((v & systemBlueMask ) >> systemBlueShift ) << 3; // B
    }
    p += JXD_SDL_SCREEN_WIDTH;
    png_write_row(png_ptr,writeBuffer);
     
    b = writeBuffer;
  }

  png_write_end(png_ptr, info_ptr);

  png_destroy_write_struct(&png_ptr, &info_ptr);

  fclose(fp);
}

void
jxd_sdl_select_font_8x8()
{
  jxd_btm        = jxd_btm_8x8;
  jxd_btm_width  = 8;
  jxd_btm_height = 8;
}

void
jxd_sdl_select_font_8x10()
{
  jxd_btm        = jxd_btm_8x10;
  jxd_btm_width  = 8;
  jxd_btm_height = 10;
}

void
jxd_sdl_select_font_8x12()
{
  jxd_btm        = jxd_btm_8x12;
  jxd_btm_width  = 8;
  jxd_btm_height = 12;
}

void
jxd_sdl_select_font_8x14()
{
  jxd_btm        = jxd_btm_8x14;
  jxd_btm_width  = 8;
  jxd_btm_height = 14;
}

void
jxd_sdl_select_font_8x16()
{
  jxd_btm        = jxd_btm_8x16;
  jxd_btm_width  = 8;
  jxd_btm_height = 16;
}

void
jxd_sdl_select_font_6x10()
{
  jxd_btm        = jxd_btm_6x10;
  jxd_btm_width  = 6;
  jxd_btm_height = 10;
}



int
jxd_sdl_init(void)
{
  if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) < 0) {
     return 0;
  }
  SDL_JoystickEventState(SDL_ENABLE);
  SDL_JoystickOpen(0);

  if (SDL_InitSubSystem(SDL_INIT_VIDEO) < 0) {
    return 0;
  }

# ifdef PND_MODE
  back_surface=SDL_SetVideoMode(JXD_SDL_SCREEN_WIDTH,JXD_SDL_SCREEN_HEIGHT, 16 , SDL_SWSURFACE|SDL_FULLSCREEN);
# else
  back_surface=SDL_SetVideoMode(JXD_SDL_SCREEN_WIDTH,JXD_SDL_SCREEN_HEIGHT, 16 , SDL_SWSURFACE);
# endif

  if ( !back_surface) {
    return 0;
  }

  jxd_sdl_init_ttf();
  jxd_sdl_select_font_tt( jxd_ttf_size );

# if !defined(LINUX_MODE)
  SDL_ShowCursor(SDL_DISABLE);
# endif

  return 1;
}

void
jxd_sdl_exit(int status)
{
  SDL_Quit();
  exit(status);
}

